@@ -1001,20 +1001,15 @@ impl<'activation> Activation<'activation> {
         });
     }
 
-    fn enqueue_for_myself_at_commit(&mut self, action: Action) {
-        let mailbox = self.state.mailbox();
-        self.pending.queue_for_mailbox(&mailbox).push(action);
-    }
-
     /// Schedule the creation of a new actor when the Activation commits.
     pub fn spawn<F: 'static + Send + FnOnce(&mut Activation) -> ActorResult>(
         &mut self,
         name: tracing::Span,
         boot: F,
     ) -> ActorRef {
-        let ac = Actor::new();
+        let ac = Actor::new(Some(self.state.actor_id));
         let ac_ref = ac.ac_ref.clone();
-        self.enqueue_for_myself_at_commit(Box::new(move |_| {
+        self.pending.for_myself.push(Box::new(move |_| {
             ac.boot(name, boot);
             Ok(())
         }));
@@ -1031,10 +1026,10 @@ impl<'activation> Activation<'activation> {
         name: tracing::Span,
         boot: F,
     ) -> ActorRef {
-        let ac = Actor::new();
+        let ac = Actor::new(Some(self.state.actor_id));
         let ac_ref = ac.ac_ref.clone();
         let facet_id = self.facet.facet_id;
-        self.enqueue_for_myself_at_commit(Box::new(move |t| {
+        self.pending.for_myself.push(Box::new(move |t| {
             t.with_facet(true, facet_id, move |t| {
                 ac.link(t).boot(name, boot);
                 Ok(())
@@ -1118,9 +1113,17 @@ impl<'activation> Activation<'activation> {
         self.stop_facet(self.facet.facet_id)
     }
 
+    /// Arranges for the active actor's root facet to be stopped cleanly when `self` commits;
+    /// this is one way to arrange a clean shutdown of the entire actor.
+    ///
+    /// Equivalent to `self.stop_facet(self.state.root)`.
+    pub fn stop_root(&mut self) {
+        self.stop_facet(self.state.root);
+    }
+
     fn stop_if_inert(&mut self) {
         let facet_id = self.facet.facet_id;
-        self.enqueue_for_myself_at_commit(Box::new(move |t| {
+        self.pending.for_myself.push(Box::new(move |t| {
             tracing::trace!("Checking inertness of facet {} from facet {}", facet_id, t.facet.facet_id);
             if t.state.facet_exists_and_is_inert(facet_id) {
                 tracing::trace!(" - facet {} is inert, stopping it", facet_id);
@@ -1166,8 +1169,7 @@ impl<'activation> Activation<'activation> {
                             tracing::trace!("not terminating parent {:?} of facet {:?}", p, facet_id);
                         }
                     } else {
-                        tracing::trace!("terminating actor of root facet {:?}", facet_id);
-                        t.state.shutdown();
+                        tracing::trace!("no parent of root facet {:?} to terminate", facet_id);
                     }
                 } else {
                     f.retract_outbound(t);
@@ -1304,14 +1306,25 @@ impl<'activation> Activation<'activation> {
     }
 
     fn restore_invariants(&mut self, d: RunDisposition) -> RunDisposition {
-        match d {
+        let d = match d {
             RunDisposition::Continue =>
                 self._restore_invariants().into(),
             RunDisposition::Terminate(Ok(())) =>
                 RunDisposition::Terminate(self._restore_invariants()),
             RunDisposition::Terminate(Err(_)) =>
                 d,
+        };
+
+        // If we would otherwise continue, check the root facet: is it still alive?
+        // If not, then the whole actor should terminate now.
+        if let RunDisposition::Continue = d {
+            if let None = self.state.get_facet(self.state.root) {
+                tracing::trace!("terminating actor because root facet no longer exists");
+                return RunDisposition::Terminate(Ok(()));
+            }
         }
+
+        d
     }
 }
 
@@ -1490,11 +1503,11 @@ impl Drop for Mailbox {
 
 impl Actor {
     /// Create a new actor. It still needs to be [`boot`ed][Self::boot].
-    pub fn new() -> Self {
+    pub fn new(parent_actor_id: Option<ActorId>) -> Self {
         let (tx, rx) = unbounded_channel();
         let actor_id = next_actor_id();
         let root = Facet::new(None);
-        tracing::trace!(?actor_id, root_facet_id = ?root.facet_id, "Actor::new");
+        tracing::debug!(?actor_id, ?parent_actor_id, root_facet_id = ?root.facet_id, "Actor::new");
         let mut st = RunningActor {
             actor_id,
             tx,
@@ -1798,7 +1811,10 @@ impl<T: Any + Send> Drop for Field<T> {
 impl Drop for Actor {
     fn drop(&mut self) {
         self.rx.close();
-        ACTORS.write().remove(&self.ac_ref.actor_id);
+        let _name = ACTORS.write().remove(&self.ac_ref.actor_id)
+            .map_or_else(|| crate::name!(parent: None, "DROPPED", actor_id=?self.ac_ref.actor_id),
+                         |(span, _ac_ref)| span);
+        let _scope = _name.enter();
         let mut g = self.ac_ref.state.lock();
         if let ActorState::Running(ref mut state) = *g {
             tracing::warn!(actor_id = ?self.ac_ref.actor_id, "Force-terminated by Actor::drop");
@@ -1807,7 +1823,7 @@ impl Drop for Actor {
             state.cleanup(&self.ac_ref, &exit_status);
             *g = ActorState::Terminated { exit_status };
         }
-        tracing::trace!(actor_id = ?self.ac_ref.actor_id, "Actor::drop");
+        tracing::debug!(actor_id = ?self.ac_ref.actor_id, "Actor::drop");
     }
 }
 
